﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 파일 열기 대화 상자 1
        /// </summary>
        private OpenFileDialog openFileDialog1;

        /// <summary>
        /// 파일 열기 대화 상자 2
        /// </summary>
        private OpenFileDialog openFileDialog2;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.openFileDialog1 = new OpenFileDialog();

            this.openFileDialog1.Filter   = "이미지 파일(*.webp, *.png, *.tif, *.tiff)|*.webp;*.png;*.tif;*.tiff";
            this.openFileDialog1.FileName = string.Empty;

            this.openFileDialog2 = new OpenFileDialog();

            this.openFileDialog2.Filter   = "WebP 파일(*.webp)|*.webp";
            this.openFileDialog2.FileName = string.Empty;

            Load                         += Form_Load;
            this.loadButton.Click        += loadButton_Click;
            this.saveButton.Click        += saveButton_Click;
            this.cropFlipButton.Click    += cropFlipButton_Click;
            this.thumbnailButton.Click   += thumbnailButton_Click;
            this.measureButton.Click     += measureButton_Click;
            this.informationButton.Click += informationButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            try
            {
                using(WEBPHelper webp = new WEBPHelper())
                {
                    Text += " (libwebp V" + webp.GetVersion() + ")";
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region 이미지 로드 버튼 클릭시 처리하기 - loadButton_Click(sender, e)

        /// <summary>
        /// 이미지 로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void loadButton_Click(object sender, EventArgs e)
        {
            try
            {
                if(this.openFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    string filePath = this.openFileDialog1.FileName;

                    if(Path.GetExtension(filePath) == ".webp")
                    {
                        using(WEBPHelper webp = new WEBPHelper())
                        {
                            this.pictureBox.Image = webp.Load(filePath);
                        }
                    }
                    else
                    {
                        this.pictureBox.Image = Image.FromFile(filePath);
                    }
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region WEBP 저장 버튼 클릭시 처리하기 - saveButton_Click(sender, e)

        /// <summary>
        /// WEBP 저장 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void saveButton_Click(object sender, EventArgs e)
        {
            if(this.pictureBox.Image == null)
            {
                MessageBox.Show("저장할 이미지가 없습니다.");

                return;
            }

            CheckForIllegalCrossThreadCalls = false;

            byte[] targetByteArray;

            try
            {
                Bitmap sourceBitmap = this.pictureBox.Image as Bitmap;

                // 손실 인코딩
                string filePath1 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SimpleLossy.webp");

                using(WEBPHelper webp = new WEBPHelper())
                {
                    targetByteArray = webp.EncodeLossy(sourceBitmap, 75);
                }

                File.WriteAllBytes(filePath1, targetByteArray);

                // 무손실 인코딩
                string filePath2 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "SimpleLossless.webp");

                using(WEBPHelper webp = new WEBPHelper())
                {
                    targetByteArray = webp.EncodeLossless(sourceBitmap);
                }

                File.WriteAllBytes(filePath2, targetByteArray);

                // 개선된 손실 인코딩
                string filePath3 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "AdvanceLossy.webp");

                using(WEBPHelper webp = new WEBPHelper())
                {
                    targetByteArray = webp.EncodeLossy(sourceBitmap, 71, 9, true);
                }

                File.WriteAllBytes(filePath3, targetByteArray);

                // 개선된 무손실 인코딩
                string filePath4 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "AdvanceLossless.webp");

                using(WEBPHelper webp = new WEBPHelper())
                {
                    targetByteArray = webp.EncodeLossless(sourceBitmap, 9);
                }

                File.WriteAllBytes(filePath4, targetByteArray);

                // 메모리에서 품질 40 및 속도 9로 근접 무손실 모드로 인코딩 테스트
                // 품질 100      : 손실 없음(-lossless와 동일한 비트 스트림)
                // 품질 80       : 매우 매우 높은 PSNR(약 54dB)이며 WebP 무손실 이미지에 비해 추가로 5-10% 크기 감소를 얻는다.
                // 품질 60       : PSNR이 매우 높고(약 48dB) WebP 무손실 이미지에 비해 추가로 20%-25% 크기 감소를 얻는다.
                // 품질 40       : 높은 PSNR(약 42dB) 및 WebP 무손실 이미지에 비해 추가로 30-35% 크기 감소를 얻는다.
                // 품질 20(이하) : 중간 PSNR(약 36dB)이며 WebP 무손실 이미지에 비해 추가로 40-50% 크기 감소를 얻는다.
                string filePath5 = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "NearLossless.webp");

                using(WEBPHelper webp = new WEBPHelper())
                {
                    targetByteArray = webp.EncodeNearLossless(sourceBitmap, 40, 9);
                }

                File.WriteAllBytes(filePath5, targetByteArray);

                MessageBox.Show("작업을 완료했습니다.");
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region 로드(CROP/FLIP) 버튼 클릭시 처리하기 - cropFlipButton_Click(sender, e)

        /// <summary>
        /// 로드(CROP/FLIP) 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cropFlipButton_Click(object sender, EventArgs e)
        {
            try
            {
                if(this.openFileDialog2.ShowDialog() == DialogResult.OK)
                {
                    string filePath = this.openFileDialog2.FileName;

                    byte[] sourceByteArray = File.ReadAllBytes(filePath);

                    WEBPDecoderOptions decoderOptions = new WEBPDecoderOptions
                    {
                        UseCropping = 1,
                        CropTop     = 10,  // 자르기 영역의 맨 위 시작
                        CropLeft    = 10,  // 자르기 영역의 왼쪽 시작
                        CropHeight  = 250, // 자르기 영역의 높이
                        CropWidth   = 300, // 자르기 영역의 너비
                        UseThreads  = 1,   // 멀티 스레딩 사용
                        Flip         = 1    // 이미지 뒤집기
                    };

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        this.pictureBox.Image = webp.Decode(sourceByteArray, decoderOptions);
                    }
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region 썸네일 로드 버튼 클릭시 처리하기 - thumbnailButton_Click(sender, e)

        /// <summary>
        /// 썸네일 로드 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void thumbnailButton_Click(object sender, EventArgs e)
        {
            try
            {
                if(this.openFileDialog2.ShowDialog() == DialogResult.OK)
                {
                    string filePath = this.openFileDialog2.FileName;

                    byte[] sourceByteArray = File.ReadAllBytes(filePath);

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        this.pictureBox.Image = webp.GetThumbnailWithSlowHighQuality(sourceByteArray, 200, 150);
                    }
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region WEBP 측정 버튼 클릭시 처리하기 - measureButton_Click(sender, e)

        /// <summary>
        /// WEBP 측정 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void measureButton_Click(object sender, EventArgs e)
        {
            try
            {
                if(this.pictureBox.Image == null)
                {
                    MessageBox.Show("이미지가 없습니다.");
                }

                if(this.openFileDialog2.ShowDialog() == DialogResult.OK)
                {
                    Bitmap  sourceBitmap = this.pictureBox.Image as Bitmap;
                    Bitmap  referenceBitmap;
                    float[] targetArray;

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        referenceBitmap = webp.Load(this.openFileDialog2.FileName);
                    }

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        targetArray = webp.GetPictureDistortion(sourceBitmap, referenceBitmap, 0); // PSNR 측정
                    }

                    MessageBox.Show
                    (
                        this,
                        $"적색 : {targetArray[0]}dB.\n" +
                        $"녹색 : {targetArray[1]}dB.\n" + 
                        $"청색 : {targetArray[2]}dB.\n" +
                        $"알파 : {targetArray[3]}dB.\n" +
                        $"전체 : {targetArray[4]}dB.",
                        "PSNR",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                    );

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        targetArray = webp.GetPictureDistortion(sourceBitmap, referenceBitmap, 1); // SSIM 측정
                    }

                    MessageBox.Show
                    (
                        this,
                        $"적색 : {targetArray[0]}dB.\n" +
                        $"녹색 : {targetArray[1]}dB.\n" + 
                        $"청색 : {targetArray[2]}dB.\n" +
                        $"알파 : {targetArray[3]}dB.\n" +
                        $"전체 : {targetArray[4]}dB.",
                        "PSNR",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                    );

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        targetArray = webp.GetPictureDistortion(sourceBitmap, referenceBitmap, 2); // LSIM 측정
                    }

                    MessageBox.Show
                    (
                        this,
                        $"적색 : {targetArray[0]}dB.\n" +
                        $"녹색 : {targetArray[1]}dB.\n" + 
                        $"청색 : {targetArray[2]}dB.\n" +
                        $"알파 : {targetArray[3]}dB.\n" +
                        $"전체 : {targetArray[4]}dB.",
                        "PSNR",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                    );
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
        #region WEBP 정보 버튼 클릭시 처리하기 - informationButton_Click(sender, e)

        /// <summary>
        /// WEBP 정보 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void informationButton_Click(object sender, EventArgs e)
        {
            int    width;
            int    height;
            bool   has_alpha;
            bool   has_animation;
            string format;

            try
            {
                if(this.openFileDialog2.ShowDialog() == DialogResult.OK)
                {
                    string filePath = this.openFileDialog2.FileName;

                    byte[] sourceByteArray = File.ReadAllBytes(filePath);

                    using(WEBPHelper webp = new WEBPHelper())
                    {
                        webp.GetInformation(sourceByteArray, out width, out height, out has_alpha, out has_animation, out format);
                    }

                    MessageBox.Show
                    (
                        this,
                        $"너비 : {width}\n"                    +
                        $"높이 : {height}\n"                   +
                        $"투명도 여부 : {has_alpha}\n"         +
                        $"애니메이션 여부 : {has_animation}\n" +
                        $"포맷 : {format}",
                        "INFORMATION",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information
                    );
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show(this, exception.ToString(), "INFORMATION", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        #endregion
    }
}