﻿using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// WEBP 보조 통계
    /// </summary>
    /// <remarks>보조 통계를 저장하기 위한 구조(대부분 손실 인코딩용)</remarks>
    [StructLayoutAttribute(LayoutKind.Sequential)]
    public struct WEBPAuxiliaryStatistics
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 코드된 크기
        /// </summary>
        /// <remarks>최종 크기</remarks>
        public int CodedSize;

        /// <summary>PSNR Y</summary>
        /// <remarks>Y에 대한 피크 신호 대 잡음비</remarks>
        public float PSNRY;

        /// <summary>
        /// PSNR U
        /// </summary>
        /// <remarks>U의 피크 신호 대 잡음비</remarks>
        public float PSNRU;

        /// <summary>
        /// PSNR V
        /// </summary>
        /// <remarks>V에 대한 피크 신호 대 잡음비</remarks>
        public float PSNRV;

        /// <summary>PSNR ALL</summary>
        /// <remarks>모두에 대한 피크 신호 대 잡음비</remarks>
        public float PSNRALL;

        /// <summary>
        /// PSNR 알파
        /// </summary>
        /// <remarks>Alpha의 피크 신호 대 잡음비</remarks>
        public float PSNRAlpha;

        /// <summary>블럭 수 인트라 4</summary>
        /// <remarks>인트라 4의 수</remarks>
        public int BlockCountIntra4;

        /// <summary>블럭 수 인트라 16</summary>
        /// <remarks>인트라 16의 수</remarks>
        public int BlockCountIntra16;

        /// <summary>블럭 수 스킵</summary>
        /// <remarks>건너뛴 매크로 블록 수</remarks>
        public int BlockCountSkipped;

        /// <summary>
        /// 헤더 바이트 수
        /// </summary>
        /// <remarks>헤더에 소요된 대략적인 바이트 수</remarks>
        public int HeaderByteCount;

        /// <summary>
        /// 모드 파티션 #0
        /// </summary>
        /// <remarks>모드 파티션 #0에 소요된 대략적인 바이트 수</remarks>
        public int ModePartition0;

        /// <summary>
        /// DC 세그먼트 0 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 0의 DC 계수에 사용된 대략적인 바이트 수</remarks>
        public int ResidualByteCountDCSegments0;

        /// <summary>
        /// AC 세그먼트 0 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 0의 AC 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountACSegments0;

        /// <summary>
        /// UV 세그먼트 0 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 0의 UV 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountUVSegments0;

        /// <summary>
        /// DC 세그먼트 1 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 1의 DC 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountDCSegments1;

        /// <summary>
        /// AC 세그먼트 1 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 1의 AC 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountACSegments1;

        /// <summary>
        /// UV 세그먼트 1 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 1의 UV 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountUVSegments1;

        /// <summary>
        /// DC 세그먼트 2 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 2의 DC 계수에 사용된 대략적인 바이트 수</remarks>
        public int ResidualByteCountDCSegments2;

        /// <summary>
        /// AC 세그먼트 2 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 2의 AC 계수에 사용된 대략적인 바이트 수</remarks>
        public int ResidualByteCountACSegments2;

        /// <summary>
        /// UV 세그먼트 2 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 2의 UV 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountUVSegments2;

        /// <summary>
        /// DC 세그먼트 3 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 3의 DC 계수에 사용된 대략적인 바이트 수</remarks>
        public int ResidualByteCountDCSegments3;

        /// <summary>
        /// AC 세그먼트 3 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 3의 AC 계수에 사용된 대략적인 바이트 수</remarks>
        public int ResidualByteCountACSegments3;

        /// <summary>
        /// UV 세그먼트 3 잔여 바이트 수
        /// </summary>
        /// <remarks>세그먼트 3의 UV 계수에 소비된 대략적인 바이트 수</remarks>
        public int ResidualByteCountUVSegments3;

        /// <summary>
        /// 세그먼트 0 세그먼트 크기
        /// </summary>
        /// <remarks>세그먼트 0의 매크로 블록 수</remarks>
        public int SegmentSizeSegments0;

        /// <summary>
        /// 세그먼트 1 세그먼트 크기
        /// </summary>
        /// <remarks>세그먼트 1의 매크로 블록 수</remarks>
        public int SegmentSizeSegments1;

        /// <summary>
        /// 세그먼트 2 세그먼트 크기
        /// </summary>
        /// <remarks>세그먼트 2의 매크로 블록 수</remarks>
        public int SegmentSizeSegments2;

        /// <summary>
        /// 세그먼트 3 세그먼트 크기
        /// </summary>
        /// <remarks>세그먼트 3의 매크로 블록 수</remarks>
        public int SegmentSizeSegments3;

        /// <summary>
        /// 세그먼트 0 세그먼트 양자화
        /// </summary>
        /// <remarks>세그먼트 0의 양자화 값</remarks>
        public int SegmentQuantizerSegments0;

        /// <summary>
        /// 세그먼트 1 세그먼트 양자화
        /// </summary>
        /// <remarks>세그먼트 1의 양자화 값</remarks>
        public int SegmentQuantizerSegments1;

        /// <summary>
        /// 세그먼트 2 세그먼트 양자화
        /// </summary>
        /// <remarks>세그먼트 2의 양자화 값</remarks>
        public int SegmentQuantizerSegments2;

        /// <summary>
        /// 세그먼트 3 세그먼트 양자화
        /// </summary>
        /// <remarks>세그먼트 3의 양자화 값</remarks>
        public int SegmentQuantizerSegments3;

        /// <summary>
        /// 세그먼트 0 세그먼트 레벨
        /// </summary>
        /// <remarks>세그먼트 0의 필터링 강도 [0..63]</remarks>
        public int SegmentLevelSegments0;

        /// <summary>
        /// 세그먼트 1 세그먼트 레벨
        /// </summary>
        /// <remarks>세그먼트 1의 필터링 강도 [0..63]</remarks>
        public int SegmentLevelSegments1;

        /// <summary>
        /// 세그먼트 2 세그먼트 레벨
        /// </summary>
        /// <remarks>세그먼트 2의 필터링 강도 [0..63]</remarks>
        public int SegmentLevelSegments2;

        /// <summary>
        /// 세그먼트 3 세그먼트 레벨
        /// </summary>
        /// <remarks>세그먼트 3의 필터링 강도 [0..63]</remarks>
        public int SegmentLevelSegments3;

        /// <summary>
        /// 알파 데이터 크기
        /// </summary>
        /// <remarks>투명도 데이터의 크기</remarks>
        public int AlphaDataSize;

        /// <summary>
        /// 레이어 데이터 크기
        /// </summary>
        /// <remarks>향상 계층 데이터의 크기</remarks>
        public int LayerDataSize;

        /// <summary>
        /// 무손실 특징
        /// </summary>
        /// <remarks>
        /// 무손실 인코더 통계
        /// bit0 : predictor, bit1 : cross-color transform, bit2 : subtract-green, bit3 : color indexing
        /// </remarks>
        public int LosslessFeatures;

        /// <summary>
        /// 히스토그램 비트 수
        /// </summary>
        /// <remarks>히스토그램의 정밀도 비트 수</remarks>
        public int HistogramBitCount;

        /// <summary>
        /// 변환 비트 수
        /// </summary>
        /// <remarks>변환용 정밀도 비트 수</remarks>
        public int TransformBitCount;

        /// <summary>
        /// 캐시 비트 수
        /// </summary>
        /// <remarks>색상 캐시 조회를 위한 비트 수</remarks>
        public int CacheBitCount;

        /// <summary>
        /// 팔레트 크기
        /// </summary>
        /// <remarks>사용된 경우 팔레트의 색상 수</remarks>
        public int PaletteSize;

        /// <summary>
        /// 무손실 크기
        /// </summary>
        /// <remarks>최종 무손실 크기</remarks>
        public int LosslessSize;

        /// <summary>
        /// 무손실 HDR 크기
        /// </summary>
        /// <remarks>무손실 헤더(transform, Huffman 등) 크기</remarks>
        public int LosslessHDRSize;

        /// <summary>
        /// 무손실 데이터 크기
        /// </summary>
        /// <remarks>무손실 이미지 데이터 크기</remarks>
        public int LosslessDataSize;

        /// <summary>
        /// 패딩
        /// </summary>
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 2, ArraySubType = UnmanagedType.U4)]
        private readonly uint[] Padding;

        #endregion
    };
}