﻿using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// WEBP 구성
    /// </summary>
    [StructLayoutAttribute(LayoutKind.Sequential)]
    public struct WEBPConfiguration
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 무손실 인코딩
        /// </summary>
        /// <remarks>무손실 인코딩 (0 : 손실(디폴트), 1 : 무손실)</remarks>
        public int Lossless;

        /// <summary>
        /// 품질
        /// </summary>
        /// <remarks>0 (가장 작은 파일) ~ 100 (가장 큰 파일)</remarks>
        public float Quality;

        /// <summary>
        /// 메소드
        /// </summary>
        /// <remarks>Quality/speed trade-off (0=fast, 6=slower-better)</remarks>
        public int Method;

        /// <summary>
        /// 이미지 힌트
        /// </summary>
        public WEBPImageHint ImageHint;

        /// <summary>
        /// 타겟 크기
        /// </summary>
        /// <remarks>
        /// 0이 아닌 경우 원하는 대상 크기를 바이트 단위로 설정한다.
        /// '압축' 매개변수보다 우선한다.
        /// </remarks>
        public int TargetSize;

        /// <summary>
        /// 타겟 PSNR
        /// </summary>
        /// <remarks>
        /// 0이 아닌 경우 달성하려는 최소 왜곡을 지정한다.
        /// TargetSize보다 우선한다.
        /// </remarks>
        public float TargetPSNR;

        /// <summary>
        /// 세그먼트 수
        /// </summary>
        /// <remarks>[1..4]에서 사용할 최대 세그먼트 수</remarks>
        public int SegmentCount;

        /// <summary>
        /// 공간 노이즈 쉐이핑 강도
        /// </summary>
        /// <remarks>공간 노이즈 쉐이핑 : 0 = 해제, 100 = 최대</remarks>
        public int SpatialNoiseShapingStrength;

        /// <summary>
        /// 필터 강도
        /// </summary>
        /// <remarks>범위 : [0 = 꺼짐 .. 100 = 가장 강함]</remarks>
        public int FilterStrength;

        /// <summary>
        /// 필터 선명도
        /// </summary>
        /// <remarks>범위 : [0 = 꺼짐 .. 7 = 최소 선명도]</remarks>
        public int FilterSharpness;

        /// <summary>
        /// 필터 타입
        /// </summary>
        /// <remarks>필터링 타입 : 0 = 단순, 1 = 강력(filter_strength > 0 또는 자동 필터 > 0인 경우에만 사용됨)</remarks>
        public int FilterType;

        /// <summary>
        /// 자동 필터
        /// </summary>
        /// <remarks>필터 강도 자동 조정 [0 = 끄기, 1 = 켜기]</remarks>
        public int AutoFilter;

        /// <summary>
        /// 알파 압축
        /// </summary>
        /// <remarks>
        /// 알파 평면 인코딩 알고리즘 (0 = 없음, 1 = WebP 무손실 압축)
        /// 기본값은 1입니다.
        /// </remarks>
        public int AlphaCompression;

        /// <summary>
        /// 알파 필터링
        /// </summary>
        /// <remarks>
        /// 알파 평면에 대한 예측 필터링 방법
        /// 0 : 없음, 1 : 빠름, 2 : 최상
        /// 1인 경우 기본값
        /// </remarks>
        public int AlphaFiltering;

        /// <summary>
        /// 알파 품질
        /// </summary>
        /// <remarks>
        /// 0(가장 작은 크기)에서 100(무손실) 사이
        /// 기본값은 100입니다.
        /// </remarks>
        public int AlphaQuality;

        /// <summary>
        /// 패스
        /// </summary>
        /// <remarks>엔트로피 분석 패스 수([1..10])</remarks>
        public int Pass;

        /// <summary>
        /// 압푹 표시
        /// </summary>
        /// <remarks>
        /// true이면 압축된 그림을 다시 내보낸다.
        /// 인루프 필터링이 적용되지 않음
        /// </remarks>
        public int ShowCompressed;

        /// <summary>
        /// 사전 처리
        /// </summary>
        /// <remarks>전처리 필터 (0 = 없음, 1 = 세그먼트 스무드, 2 = 의사 무작위 디더링)</remarks>
        public int Preprocessing;

        /// <summary>
        /// 파티션
        /// </summary>
        /// <remarks>
        /// [0..3]의 Log2(토큰 파티션 수)는 보다 쉬운 점진적 디코딩을 위해 기본값이 0으로 설정된다.
        /// </remarks>
        public int Partitions;

        /// <summary>
        /// 파티션 제한
        /// </summary>
        /// <remarks>
        /// 예측 모드 코딩에 대한 512k 제한에 맞게 품질 저하 허용(0 : 저하 없음, 100 : 가능한 최대 저하)
        /// </remarks>
        public int PartitionLimit;

        /// <summary>
        /// JPEG 크기 에뮬레이트
        /// </summary>
        /// <remarks>
        /// true인 경우 JPEG 압축의 예상 출력 크기와 더 잘 일치하도록 압축 매개변수가 다시 매핑된다.
        /// 일반적으로 출력 크기는 비슷하지만 성능 저하가 더 낮다.
        /// </remarks>
        public int EmulateJPEGSize;

        /// <summary>
        /// 쓰레드 레벨
        /// </summary>
        /// <remarks>
        /// 0이 아닌 경우 다중 스레드 인코딩을 시도하고 사용한다.
        /// </remarks>
        public int ThreadLevel;

        /// <summary>
        /// 낮은 메모리
        /// </summary>
        /// <remarks>설정하면 메모리 사용량을 줄인다(그러나 CPU 사용량을 늘림).</remarks>
        public int LowMemory;

        /// <summary>
        /// 무손실 근접
        /// </summary>
        /// <remarks>
        /// 근접 무손실 인코딩 [0 = 최대 손실 .. 100 = 꺼짐(기본값)]
        /// </remarks>
        public int NearLossless;

        /// <summary>
        /// 정확도
        /// </summary>
        /// <remarks>
        /// 0이 아닌 경우 투명 영역에서 정확한 RGB 값을 유지한다.
        /// 그렇지 않으면 더 나은 압축을 위해 이 보이지 않는 RGB 정보를 버린다.
        /// 기본값은 0이다.
        /// </remarks>
        public int Exact;

        /// <summary>
        /// 델타 팔레트화
        /// </summary>
        /// <remarks>향후 무손실 기능을 위해 예약됨</remarks>
        public int DeltaPalettization;

        /// <summary>
        /// 선명한 YUV 사용 여부
        /// </summary>
        /// <remarks>필요한 경우 선명한(그리고 느린) RGB->YUV 변환을 사용한다.</remarks>
        public int UseSharpYUV;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 패딩 1
        /// </summary>
        private readonly int Padding1;

        /// <summary>
        /// 패딩 2
        /// </summary>
        private readonly int Padding2;

        #endregion
    };
}