﻿using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// WEBP 디코더 옵션
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public struct WEBPDecoderOptions
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 바이패스 필터링
        /// </summary>
        /// <remarks>
        /// true인 경우 루프 내 필터링을 건너뛴다.
        /// </remarks>
        public int BypassFiltering;

        /// <summary>
        /// 팬시 업샘플링 없음
        /// </summary>
        /// <remarks>true인 경우 더 빠른 포인트별 업샘플러를 사용한다.</remarks>
        public int NoFancyUpsampling;

        /// <summary>
        /// 잘라내기 사용 여부
        /// </summary>
        /// <remarks>true인 경우 자르기가 _first_ 적용된다.</remarks>
        public int UseCropping;

        /// <summary>
        /// 왼쪽 잘라내기
        /// </summary>
        /// <remarks>
        /// 자르기 위한 왼쪽 위치이다.
        /// 짝수 값으로 스냅된다.
        /// </remarks>
        public int CropLeft;

        /// <summary>
        /// 위쪽 잘라내기
        /// </summary>
        /// <remarks>
        /// 자르기의 최상위 위치이다.
        /// 짝수 값으로 스냅된다.
        /// </remarks>
        public int CropTop;

        /// <summary>
        /// 너비 잘라내기
        /// </summary>
        /// <remarks>자르기 영역의 너비</remarks>
        public int CropWidth;

        /// <summary>
        /// 높이 잘라내기
        /// </summary>
        /// <remarks>자르기 영역의 높이</remarks>
        public int CropHeight;

        /// <summary>
        /// 스케일링 사용 여부
        /// </summary>
        public int UseScaling;

        /// <summary>
        /// 스케일 너비
        /// </summary>
        /// <remarks>최종 너비</remarks>
        public int ScaledWidth;

        /// <summary>
        /// 스케일 높이
        /// </summary>
        /// <remarks>최종 높이</remarks>
        public int ScaledHeight;

        /// <summary>
        /// 쓰레드 사용 여부
        /// </summary>
        /// <remarks>true인 경우 다중 스레드 디코딩을 사용한다.</remarks>
        public int UseThreads;

        /// <summary>
        /// 디더링 강도
        /// </summary>
        /// <remarks>디더링 강도(0 = 끄기, 100 = 전체)</remarks>
        public int DitheringStrength;

        /// <summary>
        /// 플립
        /// </summary>
        /// <remarks>출력을 세로로 뒤집는다.</remarks>
        public int Flip;

        /// <summary>
        /// 알파 디더링 강도
        /// </summary>
        /// <remarks>[0..100]의 알파 디더링 강도</remarks>
        public int AlphaDitheringStrength;

        /// <summary>
        /// 패딩 1
        /// </summary>
        private readonly uint Padding1;

        /// <summary>
        /// 패딩 2
        /// </summary>
        private readonly uint Padding2;

        /// <summary>
        /// 패딩 3
        /// </summary>
        private readonly uint Padding3;

        /// <summary>
        /// 패딩 4
        /// </summary>
        private readonly uint Padding4;

        /// <summary>
        /// 패딩 5
        /// </summary>
        private readonly uint Padding5;

        #endregion
    };
}