﻿using System;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// WEBP 픽처
    /// </summary>
    [StructLayoutAttribute(LayoutKind.Sequential)]
    public struct WEBPPicture
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// ARGB 사용 여부
        /// </summary>
        /// <remarks>
        /// ARGB 또는 YUV 입력 중에서 인코더를 선택하기 위한 기본 플래그이다.
        /// 손실이 없는 경우에는 ARGB 입력(*argb, argb_stride)을 사용하고 손실이 없는 경우에는 YUV 입력(*y, *u, *v 등)을 사용하는 것이 좋다.
        /// </remarks>
        public int UseARGB;

        /// <summary>
        /// 색상 공간
        /// </summary>
        /// <remarks>
        /// 색 공간: 지금은 YUV420이어야 한다(=Y'CbCr).
        /// 값 = 0
        /// </remarks>
        public uint ColorSpace;

        /// <summary>
        /// 너비
        /// </summary>
        /// <remarks>그림 너비(WEBP_MAX_DIMENSION 이하)</remarks>
        public int Width;

        /// <summary>
        /// 높이
        /// </summary>
        /// <remarks>사진 높이(WEBP_MAX_DIMENSION 이하)</remarks>
        public int Height;

        /// <summary>
        /// Y
        /// </summary>
        /// <remarks>루마 평면에 대한 포인터</remarks>
        public IntPtr Y;

        /// <summary>
        /// U
        /// </summary>
        /// <remarks>크로마 U 평면에 대한 포인터</remarks>
        public IntPtr U;

        /// <summary>
        /// V
        /// </summary>
        /// <remarks>크로마 V 평면에 대한 포인터</remarks>
        public IntPtr V;

        /// <summary>
        /// Y 스트라이드
        /// </summary>
        /// <remarks>루마 스트라이드</remarks>
        public int YStride;

        /// <summary>
        /// UV 스트라이드
        /// </summary>
        /// <remarks>크로마 스트라이드</remarks>
        public int UVStride;

        /// <summary>
        /// A
        /// </summary>
        /// <remarks>알파 평면에 대한 포인터</remarks>
        public IntPtr A;

        /// <summary>
        /// A 스트라이드
        /// </summary>
        /// <remarks>알파 평면의 보폭</remarks>
        public int AStride;

        /// <summary>
        /// 패딩 1
        /// </summary>
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 2, ArraySubType = UnmanagedType.U4)]
        private readonly uint[] Padding1;

        /// <summary>
        /// ARGB
        /// </summary>
        /// <remarks>ARGB(32비트) 평면에 대한 포인터</remarks>
        public IntPtr ARGB;

        /// <summary>
        /// ARGB 스트라이드
        /// </summary>
        /// <remarks>이것은 바이트가 아닌 픽셀 단위의 스트라이드이다.</remarks>
        public int ARGBStride;

        /// <summary>
        /// 패딩 2
        /// </summary>
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 3, ArraySubType = UnmanagedType.U4)]
        private readonly uint[] Padding2;

        /// <summary>
        /// 작성자
        /// </summary>
        /// <remarks>압축된 바이트가 준비되면 저장하기 위한 바이트 방출 후크</remarks>
        public IntPtr Writer;

        /// <summary>
        /// 커스컴 포인터
        /// </summary>
        /// <remarks>작성자가 사용할 수 있다.</remarks>
        public IntPtr CustomPointer;

        /// <summary>
        /// 부가 정보 타입
        /// </summary>
        /// <remarks>
        /// 추가 정보를 위한 맵(손실 압축 모드에만 해당)
        /// 1 : intra type, 2 : segment, 3 : quant, 4 : intra-16 prediction mode, 5 : chroma prediction mode, 6 : bit cost, 7 : distortion
        /// </remarks>
        public int ExtraInformationType;

        /// <summary>
        /// 부가 정보
        /// </summary>
        /// <remarks>
        /// NULL이 아니면 ExtraInformationType에 따라 매크로 블록 맵으로 채워질 ((width + 15) / 16) * ((height + 15) / 16) 크기의 배열을 가리킨다.
        /// </remarks>
        public IntPtr ExtraInformation;

        /// <summary>
        /// 통계
        /// </summary>
        /// <remarks>
        /// 사이드 통계에 대한 포인터(NULL이 아닌 경우에만 업데이트됨)
        /// </remarks>
        public IntPtr Statistics;

        /// <summary>
        /// 에러 코드
        /// </summary>
        /// <remarks>
        /// 인코딩 중 발생한 최신 오류에 대한 오류 코드
        /// </remarks>
        public uint ErrorCode;

        /// <summary>
        /// 프로세스 후크
        /// </summary>
        /// <remarks>NULL이 아니면 인코딩 중 진행 상황 보고</remarks>
        public IntPtr ProgressHook;

        /// <summary>
        /// 사용자 데이터
        /// </summary>
        /// <remarks>이 필드는 임의의 값으로 자유롭게 설정할 수 있으며 콜백 중에 사용된다(예 : 진행 보고서).</remarks>
        public IntPtr UserData;

        /// <summary>
        /// 패딩 3
        /// </summary>
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 13, ArraySubType = UnmanagedType.U4)]
        private readonly uint[] Padding3;

        /// <summary>
        /// 메모리 YUVA
        /// </summary>
        /// <remarks>
        /// YUVA 평면에 대한 메모리 행 청크
        /// </remarks>
        private readonly IntPtr MemoryYUVA;

        /// <summary>
        /// 메모리 ARGB
        /// </summary>
        /// <remarks>
        /// ARGB 평면에 대한 메모리 행 청크
        /// </remarks>
        private readonly IntPtr MemoryARGB;

        /// <summary>
        /// 패딩 4
        /// </summary>
        [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = 2, ArraySubType = UnmanagedType.U4)]
        private readonly uint[] Padding4;

        #endregion
    };
}