﻿using System;
using System.Runtime.InteropServices;
using System.Security;

namespace TestProject
{
    /// <summary>
    /// WIN32 헬퍼
    /// </summary>
    [SuppressUnmanagedCodeSecurity]
    public sealed partial class WIN32Helper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Delegate
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region WEBP 메모리 쓰기 대리자 - WEBPMemoryWriteDelegate(dataHandle, dataSize, picture)

        /// <summary>
        /// WEBP 메모리 쓰기 대리자
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="picture">WEBP 픽처</param>
        /// <returns>처리 결과</returns>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate int WEBPMemoryWriteDelegate([In()] IntPtr dataHandle, UIntPtr dataSize, ref WEBPPicture picture);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 메모리 복사하기 - CopyMemory(targetHandle, sourceHandle, count)

        /// <summary>
        /// 메모리 복사하기
        /// </summary>
        /// <param name="targetHandle">타겟 핸들</param>
        /// <param name="sourceHandle">소스 핸들</param>
        /// <param name="count">카운트</param>
        [DllImport("kernel32", EntryPoint = "CopyMemory", SetLastError = false)]
        public static extern void CopyMemory(IntPtr targetHandle, IntPtr sourceHandle, uint count);

        #endregion

        #region WEBP 구성 초기화하기 (내부용, X86) - WebPConfigInitInternal_x86(configuration, preset, quality, version)

        /// <summary>
        /// WEBP 구성 초기화하기 (내부용, X86)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="preset">WEBP 프리셋</param>
        /// <param name="quality">품질</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPConfigInitInternal")]
        private static extern int WebPConfigInitInternal_x86(ref WEBPConfiguration configuration, WEBPPreset preset, float quality, int version);

        #endregion
        #region WEBP 구성 초기화하기 (내부용, X64) - WebPConfigInitInternal_x64(configuration, preset, quality, version)

        /// <summary>
        /// WEBP 구성 초기화하기 (내부용, X64)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="preset">WEBP 프리셋</param>
        /// <param name="quality">품질</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPConfigInitInternal")]
        private static extern int WebPConfigInitInternal_x64(ref WEBPConfiguration configuration, WEBPPreset preset, float quality, int version);

        #endregion
        #region WEBP 특징 구하기 (내부용, X86) - WebPGetFeaturesInternal_x86(dataHandle, dataSize, features, version)

        /// <summary>
        /// WEBP 특징 구하기 (내부용, X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="features">특징</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetFeaturesInternal")]
        private static extern VP8StatusCode WebPGetFeaturesInternal_x86
        (
            [InAttribute()] IntPtr    dataHandle,
            UIntPtr                   dataSize,
            ref WEBPBitStreamFeatures features,
            int                       version
        );

        #endregion
        #region WEBP 특징 구하기 (내부용, X64) - WebPGetFeaturesInternal_x64(dataHandle, dataSize, features, version)

        /// <summary>
        /// WEBP 특징 구하기 (내부용, X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="features">특징</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetFeaturesInternal")]
        private static extern VP8StatusCode WebPGetFeaturesInternal_x64
        (
            [InAttribute()] IntPtr    rawHandle,
            UIntPtr                   dataSize,
            ref WEBPBitStreamFeatures features,
            int                       version
        );

        #endregion
        #region WEBP 무손실 프리셋 구성하기 (X86) - WebPConfigLosslessPreset_x86(configuration, level)

        /// <summary>
        /// WEBP 무손실 프리셋 구성하기 (X86)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="level">레벨</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPConfigLosslessPreset")]
        private static extern int WebPConfigLosslessPreset_x86(ref WEBPConfiguration configuration, int level);

        #endregion
        #region WEBP 무손실 프리셋 구성하기 (X64) - WebPConfigLosslessPreset_x64(configuration, level)

        /// <summary>
        /// WEBP 무손실 프리셋 구성하기 (X64)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="level">레벨</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPConfigLosslessPreset")]
        private static extern int WebPConfigLosslessPreset_x64(ref WEBPConfiguration configuration, int level);

        #endregion
        #region WEBP 구성 검증하기 (X86) - WebPValidateConfig_x86(configuration)

        /// <summary>
        /// WEBP 구성 검증하기 (X86)
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPValidateConfig")]
        private static extern int WebPValidateConfig_x86(ref WEBPConfiguration configuration);

        #endregion
        #region WEBP 구성 검증하기 (X64) - WebPValidateConfig_x64(configuration)

        /// <summary>
        /// WEBP 구성 검증하기 (X64)
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPValidateConfig")]
        private static extern int WebPValidateConfig_x64(ref WEBPConfiguration configuration);

        #endregion
        #region WEBP 픽처 초기화하기 (내부용, X86) - WebPPictureInitInternal_x86(picture, version)

        /// <summary>
        /// WEBP 픽처 초기화하기 (내부용, X86)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureInitInternal")]
        private static extern int WebPPictureInitInternal_x86(ref WEBPPicture picture, int version);

        #endregion
        #region WEBP 픽처 초기화하기 (내부용, X64) - WebPPictureInitInternal_x64(picture, version)

        /// <summary>
        /// WEBP 픽처 초기화하기 (내부용, X64)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureInitInternal")]
        private static extern int WebPPictureInitInternal_x64(ref WEBPPicture picture, int version);

        #endregion
        #region WEBP 픽처 BGR 가져오기 (X86) - WebPPictureImportBGR_x86(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 BGR 가져오기 (X86)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGR")]
        private static extern int WebPPictureImportBGR_x86(ref WEBPPicture picture, IntPtr bgr, int stride);

        #endregion
        #region WEBP 픽처 BGR 가져오기 (X64) - WebPPictureImportBGR_x64(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 BGR 가져오기 (X64)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGR")]
        private static extern int WebPPictureImportBGR_x64(ref WEBPPicture picture, IntPtr bgr, int stride);

        #endregion
        #region WEBP 픽처 BGRA 가져오기 (X86) - WebPPictureImportBGRA_x86(picture, bgra, stride)

        /// <summary>
        /// WEBP 픽처 BGRA 가져오기 (X86)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgra">BGRA</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGRA")]
        private static extern int WebPPictureImportBGRA_x86(ref WEBPPicture picture, IntPtr bgra, int stride);

        #endregion
        #region WEBP 픽처 BGRA 가져오기 (X64) - WebPPictureImportBGRA_x64(picture, bgra, stride)

        /// <summary>
        /// WEBP 픽처 BGRA 가져오기 (X64)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgra">BGRA</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGRA")]
        private static extern int WebPPictureImportBGRA_x64(ref WEBPPicture picture, IntPtr bgra, int stride);

        #endregion
        #region WEBP 픽처 BGRX 가져오기 (X86) - WebPPictureImportBGRX_x86(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 BGRX 가져오기 (X86)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGRX")]
        private static extern int WebPPictureImportBGRX_x86(ref WEBPPicture picture, IntPtr bgr, int stride);

        #endregion
        #region WEBP 픽처 BGRX 가져오기 (X64) - WebPPictureImportBGRX_x64(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 BGRX 가져오기 (X64)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureImportBGRX")]
        private static extern int WebPPictureImportBGRX_x64(ref WEBPPicture picture, IntPtr bgr, int stride);

        #endregion
        #region WEBP 인코딩하기 (X86) - WebPEncode_x86(configuration, picture)

        /// <summary>
        /// WEBP 인코딩하기 (X86)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="picture">WEBP 픽처</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncode")]
        private static extern int WebPEncode_x86(ref WEBPConfiguration configuration, ref WEBPPicture picture);

        #endregion
        #region WEBP 인코딩하기 (X64) - WebPEncode_x64(configuration, picture)

        /// <summary>
        /// WEBP 인코딩하기 (X64)
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="picture">WEBP 픽처</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncode")]
        private static extern int WebPEncode_x64(ref WEBPConfiguration configuration, ref WEBPPicture picture);

        #endregion
        #region WEBP 픽처 해제하기 (X86) - WebPPictureFree_x86(picture)

        /// <summary>
        /// WEBP 픽처 해제하기 (X86)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureFree")]
        private static extern void WebPPictureFree_x86(ref WEBPPicture picture);

        #endregion
        #region WEBP 픽처 해제하기 (X64) - WebPPictureFree_x64(picture)

        /// <summary>
        /// WEBP 픽처 해제하기 (X64)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureFree")]
        private static extern void WebPPictureFree_x64(ref WEBPPicture picture);

        #endregion
        #region WEBP 정보 구하기 (X86) - WebPGetInfo_x86(dataHandle, dataSize, width, height)

        /// <summary>
        /// WEBP 정보 구하기 (X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetInfo")]
        private static extern int WebPGetInfo_x86([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, out int width, out int height);

        #endregion
        #region WEBP 정보 구하기 (X64) - WebPGetInfo_x64(dataHandle, dataSize, width, height)

        /// <summary>
        /// WEBP 정보 구하기 (X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetInfo")]
        private static extern int WebPGetInfo_x64([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, out int width, out int height);

        #endregion
        #region WEBP BGR 디코딩하기 (X86) - WebPDecodeBGRInto_x86(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGR 디코딩하기 (X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeBGRInto")]
        private static extern IntPtr WebPDecodeBGRInto_x86([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP BGR 디코딩하기 (X64) - WebPDecodeBGRInto_x64(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGR 디코딩하기 (X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeBGRInto")]
        private static extern IntPtr WebPDecodeBGRInto_x64([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP BGRA 디코딩하기 (X86) - WebPDecodeBGRAInto_x86(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGRA 디코딩하기 (X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeBGRAInto")]
        private static extern IntPtr WebPDecodeBGRAInto_x86([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP BGRA 디코딩하기 (X64) - WebPDecodeBGRAInto_x64(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGRA 디코딩하기 (X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeBGRAInto")]
        private static extern IntPtr WebPDecodeBGRAInto_x64([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP ARGB 디코딩하기 (X86) - WebPDecodeARGBInto_x86(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP ARGB 디코딩하기 (X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeARGBInto")]
        private static extern IntPtr WebPDecodeARGBInto_x86([InAttribute()] IntPtr dataHandle, UIntPtr dataSize, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP ARGB 디코딩하기 (X64) - WebPDecodeARGBInto_x64(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP ARGB 디코딩하기 (X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecodeARGBInto")]
        private static extern IntPtr WebPDecodeARGBInto_x64([InAttribute()] IntPtr data, UIntPtr data_size, IntPtr bufferHandle, int bufferSize, int stride);

        #endregion
        #region WEBP 디코더 구성 초기화하기 (X86) - WebPInitDecoderConfigInternal_x86(configuration, version)

        /// <summary>
        /// WEBP 디코더 구성 초기화하기 (X86)
        /// </summary>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPInitDecoderConfigInternal")]
        private static extern int WebPInitDecoderConfigInternal_x86(ref WEBPDecoderConfiguration configuration, int version);

        #endregion
        #region WEBP 디코더 구성 초기화하기 (X64) - WebPInitDecoderConfigInternal_x64(configuration, version)

        /// <summary>
        /// WEBP 디코더 구성 초기화하기 (X64)
        /// </summary>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <param name="version">버전</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPInitDecoderConfigInternal")]
        private static extern int WebPInitDecoderConfigInternal_x64(ref WEBPDecoderConfiguration configuration, int version);

        #endregion
        #region WEBP 디코딩하기 (X86) - WebPDecode_x86(dataHandle, dataSize, configuration)

        /// <summary>
        /// WEBP 디코딩하기 (X86)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <returns>상태 코드</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecode")]
        private static extern VP8StatusCode WebPDecode_x86(IntPtr dataHandle, UIntPtr dataSize, ref WEBPDecoderConfiguration configuration);

        #endregion
        #region WEBP 디코딩하기 (X64) - WebPDecode_x64(dataHandle, dataSize, configuration)

        /// <summary>
        /// WEBP 디코딩하기 (X64)
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <returns>상태 코드</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPDecode")]
        private static extern VP8StatusCode WebPDecode_x64(IntPtr dataHandle, UIntPtr dataSize, ref WEBPDecoderConfiguration configuration);

        #endregion
        #region WEBP 디코더 버퍼 해제하기 (X86) - WebPFreeDecBuffer_x86(buffer)

        /// <summary>
        /// WEBP 디코더 버퍼 해제하기 (X86)
        /// </summary>
        /// <param name="buffer">WEBP 디코더 버퍼</param>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPFreeDecBuffer")]
        private static extern void WebPFreeDecBuffer_x86(ref WEBPDecoderBuffer buffer);

        #endregion
        #region WEBP 디코더 버퍼 해제하기 (X64) - WebPFreeDecBuffer_x64(buffer)

        /// <summary>
        /// WEBP 디코더 버퍼 해제하기 (X64)
        /// </summary>
        /// <param name="buffer">WEBP 디코더 버퍼</param>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPFreeDecBuffer")]
        private static extern void WebPFreeDecBuffer_x64(ref WEBPDecoderBuffer buffer);

        #endregion
        #region WEBP BGR 인코딩하기 (X86) - WebPEncodeBGR_x86(bgrHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGR 인코딩하기 (X86)
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeBGR")]
        private static extern int WebPEncodeBGR_x86([InAttribute()] IntPtr bgrHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle);

        #endregion
        #region WEBP BGR 인코딩하기 (X64) - WebPEncodeBGR_x64(bgrHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGR 인코딩하기 (X64)
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeBGR")]
        private static extern int WebPEncodeBGR_x64([InAttribute()] IntPtr bgrHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle);

        #endregion
        #region WEBP BGRA 인코딩하기 (X86) - WebPEncodeBGRA_x86(bgraHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGRA 인코딩하기 (X86)
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeBGRA")]
        private static extern int WebPEncodeBGRA_x86([InAttribute()] IntPtr bgraHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle);

        #endregion
        #region WEBP BGRA 인코딩하기 (X64) - WebPEncodeBGRA_x64(bgraHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGRA 인코딩하기 (X64)
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeBGRA")]
        private static extern int WebPEncodeBGRA_x64([InAttribute()] IntPtr bgraHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle);

        #endregion
        #region WEBP BGR 무손실 인코딩하기 (X86) - WebPEncodeLosslessBGR_x86(bgrHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGR 무손실 인코딩하기 (X86)
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeLosslessBGR")]
        private static extern int WebPEncodeLosslessBGR_x86([InAttribute()] IntPtr bgrHandle, int width, int height, int stride, out IntPtr outputHandle);

        #endregion
        #region WEBP BGR 무손실 인코딩하기 (X64) - WebPEncodeLosslessBGR_x64(bgrHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGR 무손실 인코딩하기 (X64)
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeLosslessBGR")]
        private static extern int WebPEncodeLosslessBGR_x64([InAttribute()] IntPtr bgrHandle, int width, int height, int stride, out IntPtr outputHandle);

        #endregion
        #region WEBP BGRA 무손실 인코딩하기 (X86) - WebPEncodeLosslessBGRA_x86(bgraHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGRA 무손실 인코딩하기 (X86)
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeLosslessBGRA")]
        private static extern int WebPEncodeLosslessBGRA_x86([InAttribute()] IntPtr bgraHandle, int width, int height, int stride, out IntPtr outputHandle);

        #endregion
        #region WEBP BGRA 무손실 인코딩하기 (X64) - WebPEncodeLosslessBGRA_x64(bgraHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGRA 무손실 인코딩하기 (X64)
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPEncodeLosslessBGRA")]
        private static extern int WebPEncodeLosslessBGRA_x64([InAttribute()] IntPtr bgraHandle, int width, int height, int stride, out IntPtr outputHandle);

        #endregion
        #region WEBP 해제하기 (X86) - WebPFree_x86(handle)

        /// <summary>
        /// WEBP 해제하기 (X86)
        /// </summary>
        /// <param name="handle">핸들</param>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPFree")]
        private static extern void WebPFree_x86(IntPtr handle);

        #endregion
        #region WEBP 해제하기 (X64) - WebPFree_x64(handle)

        /// <summary>
        /// WEBP 해제하기 (X64)
        /// </summary>
        /// <param name="handle">핸들</param>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPFree")]
        private static extern void WebPFree_x64(IntPtr handle);

        #endregion
        #region WEBP 디코더 버전 구하기 (X86) - WebPGetDecoderVersion_x86()

        /// <summary>
        /// WEBP 디코더 버전 구하기 (X86)
        /// </summary>
        /// <returns>WEBP 디코더 버전</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetDecoderVersion")]
        private static extern int WebPGetDecoderVersion_x86();

        #endregion
        #region WEBP 디코더 버전 구하기 (X64) - WebPGetDecoderVersion_x64()

        /// <summary>
        /// WEBP 디코더 버전 구하기 (X64)
        /// </summary>
        /// <returns>WEBP 디코더 버전</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPGetDecoderVersion")]
        private static extern int WebPGetDecoderVersion_x64();

        #endregion
        #region WEBP 픽처 왜곡 구하기 (X86) - WebPPictureDistortion_x86(sourcePicture, targetPicture, metricType, resultHandle)

        /// <summary>
        /// WEBP 픽처 왜곡 구하기 (X86)
        /// </summary>
        /// <param name="sourcePicture">소스 픽처</param>
        /// <param name="targetPicture">타겟 픽처</param>
        /// <param name="metricType">미터법 타입</param>
        /// <param name="resultHandle">결과 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x86", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureDistortion")]
        private static extern int WebPPictureDistortion_x86(ref WEBPPicture sourcePicture, ref WEBPPicture targetPicture, int metricType, IntPtr resultHandle);

        #endregion
        #region WEBP 픽처 왜곡 구하기 (X64) - WebPPictureDistortion_x64(sourcePicture, targetPicture, metricType, resultHandle)

        /// <summary>
        /// WEBP 픽처 왜곡 구하기 (X64)
        /// </summary>
        /// <param name="sourcePicture">소스 픽처</param>
        /// <param name="targetPicture">타겟 픽처</param>
        /// <param name="metricType">미터법 타입</param>
        /// <param name="resultHandle">결과 핸들</param>
        /// <returns>처리 결과</returns>
        [DllImport("libwebp_x64", CallingConvention = CallingConvention.Cdecl, EntryPoint = "WebPPictureDistortion")]
        private static extern int WebPPictureDistortion_x64(ref WEBPPicture sourcePicture, ref WEBPPicture targetPicture, int metricType, IntPtr resultHandle);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 메모리 쓰기 대리자
        /// </summary>
        public static WEBPMemoryWriteDelegate memoryWriteDelegate;

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// WEBP 디코더 ABI 버전
        /// </summary>
        private static readonly int WEBP_DECODER_ABI_VERSION = 0x0208;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region WEBP 구성 초기화하기 - WEBPInitializeConfiguration(configuration, preset, quality)

        /// <summary>
        /// WEBP 구성 초기화하기
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="preset">WEBP 프리셋</param>
        /// <param name="quality">품질</param>
        /// <returns>처리 결과</returns>
        public static int WEBPInitializeConfiguration(ref WEBPConfiguration configuration, WEBPPreset preset, float quality)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPConfigInitInternal_x86(ref configuration, preset, quality, WEBP_DECODER_ABI_VERSION);
                case 8  : return WebPConfigInitInternal_x64(ref configuration, preset, quality, WEBP_DECODER_ABI_VERSION);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 특징 구하기 - WEBPGetFeatures(dataHandle, dataSize, features)

        /// <summary>
        /// WEBP 특징 구하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="features">특징</param>
        /// <returns>처리 결과</returns>
        public static VP8StatusCode WEBPGetFeatures(IntPtr dataHandle, int dataSize, ref WEBPBitStreamFeatures features)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPGetFeaturesInternal_x86(dataHandle, (UIntPtr)dataSize, ref features, WEBP_DECODER_ABI_VERSION);
                case 8  : return WebPGetFeaturesInternal_x64(dataHandle, (UIntPtr)dataSize, ref features, WEBP_DECODER_ABI_VERSION);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 무손실 프리셋 구성하기 - WEBPConfigLosslessPreset(configuration, level)

        /// <summary>
        /// WEBP 무손실 프리셋 구성하기
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <param name="level">레벨</param>
        /// <returns>처리 결과</returns>
        public static int WEBPConfigLosslessPreset(ref WEBPConfiguration configuration, int level)
        {
            switch (IntPtr.Size)
            {
                case 4  : return WebPConfigLosslessPreset_x86(ref configuration, level);
                case 8  : return WebPConfigLosslessPreset_x64(ref configuration, level);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 구성 검증하기 - WEBPValidateConfiguration(configuration)

        /// <summary>
        /// WEBP 구성 검증하기
        /// </summary>
        /// <param name="configuration">구성</param>
        /// <returns>처리 결과</returns>
        public static int WEBPValidateConfiguration(ref WEBPConfiguration configuration)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPValidateConfig_x86(ref configuration);
                case 8  : return WebPValidateConfig_x64(ref configuration);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 초기화하기 (내부용) - WEBPInitializePictureInternal(picture)

        /// <summary>
        /// WEBP 픽처 초기화하기 (내부용)
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <returns>처리 결과</returns>
        public static int WEBPInitializePictureInternal(ref WEBPPicture picture)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPPictureInitInternal_x86(ref picture, WEBP_DECODER_ABI_VERSION);
                case 8  : return WebPPictureInitInternal_x64(ref picture, WEBP_DECODER_ABI_VERSION);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 RGB 가져오기 - WEBPPictureImportBGR(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 RGB 가져오기
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        public static int WEBPPictureImportBGR(ref WEBPPicture picture, IntPtr bgr, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPPictureImportBGR_x86(ref picture, bgr, stride);
                case 8  : return WebPPictureImportBGR_x64(ref picture, bgr, stride);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 BGRA 가져오기 - WEBPPictureImportBGRA(picture, bgra, stride)

        /// <summary>
        /// WEBP 픽처 BGRA 가져오기
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgra">BGRA</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        public static int WEBPPictureImportBGRA(ref WEBPPicture picture, IntPtr bgra, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPPictureImportBGRA_x86(ref picture, bgra, stride);
                case 8  : return WebPPictureImportBGRA_x64(ref picture, bgra, stride);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 BGRX 가져오기 - WEBPPictureImportBGRX(picture, bgr, stride)

        /// <summary>
        /// WEBP 픽처 BGRX 가져오기
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        /// <param name="bgr">BGR</param>
        /// <param name="stride">스트라이드</param>
        /// <returns>처리 결과</returns>
        public static int WEBPPictureImportBGRX(ref WEBPPicture picture, IntPtr bgr, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPPictureImportBGRX_x86(ref picture, bgr, stride);
                case 8  : return WebPPictureImportBGRX_x64(ref picture, bgr, stride);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 인코딩하기 - WEBPEncode(configuration, picture)

        /// <summary>
        /// WEBP 인코딩하기
        /// </summary>
        /// <param name="configuration">WEBP 구성</param>
        /// <param name="picture">WEBP 픽처</param>
        /// <returns>처리 결과</returns>
        /// <exception cref="InvalidOperationException"></exception>
        public static int WEBPEncode(ref WEBPConfiguration configuration, ref WEBPPicture picture)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPEncode_x86(ref configuration, ref picture);
                case 8  : return WebPEncode_x64(ref configuration, ref picture);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 해제하기 - WEBPPictureFree(picture)

        /// <summary>
        /// WEBP 픽처 해제하기
        /// </summary>
        /// <param name="picture">WEBP 픽처</param>
        public static void WEBPPictureFree(ref WEBPPicture picture)
        {
            switch(IntPtr.Size)
            {
                case 4  : WebPPictureFree_x86(ref picture); break;
                case 8  : WebPPictureFree_x64(ref picture); break;
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 정보 구하기 - WEBPGetInformation(dataHandle, dataSize, width, height)

        /// <summary>
        /// WEBP 정보 구하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <returns>처리 결과</returns>
        /// <exception cref="InvalidOperationException"></exception>
        public static int WEBPGetInformation(IntPtr dataHandle, int dataSize, out int width, out int height)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPGetInfo_x86(dataHandle, (UIntPtr)dataSize, out width, out height);
                case 8  : return WebPGetInfo_x64(dataHandle, (UIntPtr)dataSize, out width, out height);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGR 디코딩하기 - WebPDecodeBGRInto(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGR 디코딩하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        /// <exception cref="InvalidOperationException"></exception>
        public static void WebPDecodeBGRInto(IntPtr dataHandle, int dataSize, IntPtr bufferHandle, int bufferSize, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4 :

                    if(WebPDecodeBGRInto_x86(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                case 8 :

                    if(WebPDecodeBGRInto_x64(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                default :

                    throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGRA 디코딩하기 - WEBPDecodeBGRAInto(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP BGRA 디코딩하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        public static void WEBPDecodeBGRAInto(IntPtr dataHandle, int dataSize, IntPtr bufferHandle, int bufferSize, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4 :

                    if(WebPDecodeBGRAInto_x86(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                case 8 :

                    if(WebPDecodeBGRAInto_x64(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                default :

                    throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP ARGB 디코딩하기 - WEBPDecodeARGBInto(dataHandle, dataSize, bufferHandle, bufferSize, stride)

        /// <summary>
        /// WEBP ARGB 디코딩하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="bufferHandle">버퍼 핸들</param>
        /// <param name="bufferSize">버퍼 크기</param>
        /// <param name="stride">스트라이드</param>
        public static void WEBPDecodeARGBInto(IntPtr dataHandle, int dataSize, IntPtr bufferHandle, int bufferSize, int stride)
        {
            switch(IntPtr.Size)
            {
                case 4 :

                    if(WebPDecodeARGBInto_x86(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                case 8 :

                    if(WebPDecodeARGBInto_x64(dataHandle, (UIntPtr)dataSize, bufferHandle, bufferSize, stride) == null)
                    {
                        throw new InvalidOperationException("Can not decode WebP");
                    }

                    break;

                default :

                    throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 디코더 구성 초기화하기 - WEBPInitializeDecoderConfiguration(configuration)

        /// <summary>
        /// WEBP 디코더 구성 초기화하기
        /// </summary>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <returns>처리 결과</returns>
        public static int WEBPInitializeDecoderConfiguration(ref WEBPDecoderConfiguration configuration)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPInitDecoderConfigInternal_x86(ref configuration, WEBP_DECODER_ABI_VERSION);
                case 8  : return WebPInitDecoderConfigInternal_x64(ref configuration, WEBP_DECODER_ABI_VERSION);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 디코딩하기 - WEBPDecode(dataHandle, dataSize, configuration)

        /// <summary>
        /// WEBP 디코딩하기
        /// </summary>
        /// <param name="dataHandle">데이터 핸들</param>
        /// <param name="dataSize">데이터 크기</param>
        /// <param name="configuration">WEBP 디코더 구성</param>
        /// <returns>상태 코드</returns>
        public static VP8StatusCode WEBPDecode(IntPtr dataHandle, int dataSize, ref WEBPDecoderConfiguration configuration)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPDecode_x86(dataHandle, (UIntPtr)dataSize, ref configuration);
                case 8  : return WebPDecode_x64(dataHandle, (UIntPtr)dataSize, ref configuration);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 디코더 버퍼 해제하기 - WEBPFreeDecoderBuffer(buffer)

        /// <summary>
        /// WEBP 디코더 버퍼 해제하기
        /// </summary>
        /// <param name="buffer">WEBP 디코더 버퍼</param>
        public static void WEBPFreeDecoderBuffer(ref WEBPDecoderBuffer buffer)
        {
            switch(IntPtr.Size)
            {
                case 4  : WebPFreeDecBuffer_x86(ref buffer); break;
                case 8  : WebPFreeDecBuffer_x64(ref buffer); break;
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGR 인코딩하기 - WEBPEncodeBGR(bgrHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGR 인코딩하기
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        public static int WEBPEncodeBGR(IntPtr bgrHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPEncodeBGR_x86(bgrHandle, width, height, stride, qualityFactor, out outputHandle);
                case 8  : return WebPEncodeBGR_x64(bgrHandle, width, height, stride, qualityFactor, out outputHandle);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGRA 인코딩하기 - WEBPEncodeBGRA(bgraHandle, width, height, stride, qualityFactor, outputHandle)

        /// <summary>
        /// WEBP BGRA 인코딩하기
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="qualityFactor">품질 인자</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        public static int WEBPEncodeBGRA(IntPtr bgraHandle, int width, int height, int stride, float qualityFactor, out IntPtr outputHandle)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPEncodeBGRA_x86(bgraHandle, width, height, stride, qualityFactor, out outputHandle);
                case 8  : return WebPEncodeBGRA_x64(bgraHandle, width, height, stride, qualityFactor, out outputHandle);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGR 무손실 인코딩하기 - WEBPEncodeLosslessBGR(bgrHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGR 무손실 인코딩하기
        /// </summary>
        /// <param name="bgrHandle">BGR 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        public static int WEBPEncodeLosslessBGR(IntPtr bgrHandle, int width, int height, int stride, out IntPtr outputHandle)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPEncodeLosslessBGR_x86(bgrHandle, width, height, stride, out outputHandle);
                case 8  : return WebPEncodeLosslessBGR_x64(bgrHandle, width, height, stride, out outputHandle);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP BGRA 무손실 인코딩하기 - WEBPEncodeLosslessBGRA(bgraHandle, width, height, stride, outputHandle)

        /// <summary>
        /// WEBP BGRA 무손실 인코딩하기
        /// </summary>
        /// <param name="bgraHandle">BGRA 핸들</param>
        /// <param name="width">너비</param>
        /// <param name="height">높이</param>
        /// <param name="stride">스트라이드</param>
        /// <param name="outputHandle">출력 핸들</param>
        /// <returns>처리 결과</returns>
        public static int WEBPEncodeLosslessBGRA(IntPtr bgraHandle, int width, int height, int stride, out IntPtr outputHandle)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPEncodeLosslessBGRA_x86(bgraHandle, width, height, stride, out outputHandle);
                case 8  : return WebPEncodeLosslessBGRA_x64(bgraHandle, width, height, stride, out outputHandle);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 해제하기 - WEBPFree(handle)

        /// <summary>
        /// WEBP 해제하기
        /// </summary>
        /// <param name="handle">핸들</param>
        public static void WEBPFree(IntPtr handle)
        {
            switch(IntPtr.Size)
            {
                case 4  : WebPFree_x86(handle); break;
                case 8  : WebPFree_x64(handle); break;
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 디코더 버전 구하기 - WEBPGetDecoderVersion()

        /// <summary>
        /// WEBP 디코더 버전 구하기
        /// </summary>
        /// <returns>WEBP 디코더 버전</returns>
        public static int WEBPGetDecoderVersion()
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPGetDecoderVersion_x86();
                case 8  : return WebPGetDecoderVersion_x64();
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
        #region WEBP 픽처 왜곡 구하기 - WEBPPictureDistortion(sourcePicture, targetPicture, metricType, resultHandle)

        /// <summary>
        /// WEBP 픽처 왜곡 구하기
        /// </summary>
        /// <param name="sourcePicture">소스 픽처</param>
        /// <param name="targetPicture">타겟 픽처</param>
        /// <param name="metricType">미터법 타입</param>
        /// <param name="resultHandle">결과 핸들</param>
        /// <returns>처리 결과</returns>
        public static int WEBPPictureDistortion(ref WEBPPicture sourcePicture, ref WEBPPicture targetPicture, int metricType, IntPtr resultHandle)
        {
            switch(IntPtr.Size)
            {
                case 4  : return WebPPictureDistortion_x86(ref sourcePicture, ref targetPicture, metricType, resultHandle);
                case 8  : return WebPPictureDistortion_x64(ref sourcePicture, ref targetPicture, metricType, resultHandle);
                default : throw new InvalidOperationException("Invalid platform. Can not find proper function");
            }
        }

        #endregion
    }
}